/*
  game_free.c
  jigplay-macos
 
  Created by François Pessaux on 16/09/10.
  Copyright 2010. All rights reserved.
*/

#include <stdlib.h>

#include "game_free.h"


void freeLayout (struct layout_cell *layout, int nb_cells)
{
  int i ;
  
  if (layout == NULL) return ;

  /* Free the list of each layout cell. */
  for (i = 0; i < nb_cells; i++)
    freePiecesList (layout[i].hosted_pieces) ;
    
  /* Free the layout cells. */
  free (layout) ;
}



void freePieces (struct piece *pieces, int nb_pieces)
{
  int i ;
  
  if (pieces == NULL) return ;
  
  /* Free each piece's internal data. */
  for (i = 0; i < nb_pieces; i++) {
    int j ;
    struct piece *piece = &pieces[i] ;
    
    /* Free the edges splines then the edges. */
    for (j = 0; j < 4; j++) {
      if (piece->free_edges[j] != NULL) {
        if (piece->free_edges[j]->spline != NULL) {
          if (piece->free_edges[j]->spline->vertices != NULL)
            free (piece->free_edges[j]->spline->vertices) ;
          free (piece->free_edges[j]->spline) ;
        }
        free (piece->free_edges[j]) ;
      }
    }    /* End of for (j = 0; j < 4; j++). */
    
    /* Free the connected-set only if we are the last guy to retain it. In
       effect, the connected-set structure if physically shared between all
       the members of this connected set. Hence we must not free it several
       times and we must not free it as long as somebody still retains it.*/
    if (piece->connected_pieces != NULL) {
      if (piece->connected_pieces->refcount == 1) {
        freePiecesList (piece->connected_pieces->members) ;
        free (piece->connected_pieces) ;
      }
      else piece->connected_pieces->refcount-- ;
    }    /* End of if (piece->connected_pieces != NULL). */
    
    /* Free the image structure. */
    if (piece->piece_image != NULL) freeImageWithMask (piece->piece_image) ;
  }     /* End of for (i = 0; i < nb_pieces; i++). */
  
  /* Free the array containing pointers on the pieces. */
  free (pieces) ;
}



void freeGame (struct game *game)
{  
  if (game == NULL) return ;
  freeLayout (game->layout, game->layout_nb_columns * game->layout_nb_lines) ;
  freePieces (game->pieces, game->nb_pieces) ;

  /* Free the image. */
  if (game->game_image != NULL) CGImageRelease (game->game_image) ;
  
  /* Free the data of the original image file. */
  if (game->image_file_data != NULL) free (game->image_file_data) ;
  
  /* Finally, free the game structure. */
  free (game) ;
}
