/*
  game_struct.h
  jigplay-macos
 
  Created by François Pessaux on 16/09/10.
  Copyright 2010. All rights reserved.
*/

#ifndef __GAME_STRUCT_H__
#define __GAME_STRUCT_H__

#ifdef __JIGPLAY_MAC_OS__
#include <Carbon/Carbon.h>
#else
#ifdef __JIGPLAY_IOS__
#include <CoreGraphics/CGImage.h>
#endif
#endif

#include "misc.h"
#include "piece.h"


#define TAGTOLONG(a,b,c,d) ((unsigned int)(((a)<<24)|((b)<<16)|((c)<<8)|(d)))
#define SAVEMAGICNUMBER TAGTOLONG('J','I','G','P')


/** Describes each cell of the layout, organised as a bi-dimensional
    array of such cells. Each cell record the pieces whose bounding
    box spen on and a boolean flag telling if the pieces of this cell
    must be redrawn. */
struct layout_cell {
  int to_redraw ;
  struct pieces_list *hosted_pieces ;
};



struct game {
  /** Boolean telling if the game is still living because not yet won. */
  bool_t game_won ;
  /** Image width in pixels. The image width is the one really used in the
      game, i.e. possibly smaller than the original image if this latter was
      cropped to get an integer width of pieces according to the number of
      pieces requested horizontally by the user. */
  int image_width ;
  /** Image height in pixels. The image height is the one really used in the
      game, i.e. possibly smaller than the original image if this latter was
      cropped to get an integer height of pieces according to the number of
      pieces requested vertically by the user. */
  int image_height ;
  /** The scaling factor to apply to the original image. If negative of null,
      then no scaling is to apply to the image and this latter keeps it
      original size. The above image size already takes into account the
      scaling. This factor is only needed when rebuilding the image to map
      from the original picture file in case of loadign a saved game. */
  float original_image_scale_factor ;
  /** Puzzle piece base width (i.e. without deformation) in pixels. */
  int piece_width ;
  /** Puzzle piece base height (i.e. without deformation) in pixels. */
  int piece_height ;
  /** Computed to be image_width / piece_width. Represents the size
      of the layout. But we could have a layout larger then image. */
  int layout_nb_columns ;
  /** Computed to be image_height / piece_height. Represents the size
      of the layout. But we could have a layout larger then image. */
  int layout_nb_lines ;
  /** Grid telling which pieces are in which lines / columns.
      We allocate it as a global array and access it by
      (y * layout_nb_columns) + x.
      Since there can be several pieces overlapping a cell, each cell contains a
      list of pieces. Pieces are set in the layout cells based on their
      bounding box. */
  struct layout_cell *layout ;
  /** Number of pieces in the game. */
  int nb_pieces ;
  /* Array of pieces, assumed to be indexed by the pieces ids. I.e. at index
     i, we find the piece whose index is i. */
  struct piece *pieces ;
  /** Head of the double-linked sorted list of edges by increasing x. */
  struct edge *x_sorted_edges_head ;
  /** Tail of the double-linked sorted list of edges by increasing x. */
  struct edge *x_sorted_edges_tail ;
  /** Head of the double-linked sorted list of edges by increasing y. */
  struct edge *y_sorted_edges_head ;
  /** Tail of the double-linked sorted list of edges by increasing y. */
  struct edge *y_sorted_edges_tail ;
  /* Reference to the graphic object representing the complete image. */
  CGImageRef game_image ;
  /* Size in bytes of the data representing the image file. */
  int image_file_size ;
  /** Save of the content of the imge file. Allows to prevent problems in case
      of image disparition between save / load game. */
  unsigned char *image_file_data ;
};

#endif
