/*
  resolutions_suggestion.c
  jigplay-macos

  Created by François Pessaux on 11/10/10.
  Copyright 2010. All rights reserved.
*/

#include <stdlib.h>

#include "resolutions_suggestion.h"



#define MIN_PIECE_SIZE (30)



/** Computes the differents configurations of vertical and horizontal number
    of pieces according to the specified image width and height so that pieces
    are square and not smaller than MIN_PIECE_SIZE pixels (both in width and
    height).
    Creates an array containing the found possible resolutions, returns it
    (caller is in charge to free it) and set, by side effect, the number of
    found resolutions (i.e. the number of elements of the returned array) in the
    out-mode parameter nb_resolutions. */
struct resolution* computeRecommendedResolutions (int width, int height,
                                                  int *nb_resolutions)
{
  float ratio ;
  int nb_min_vert, nb_max_vert, nb_min_horiz, nb_max_horiz ;
  int i, j ;
  struct resolution *resolutions ;

  if (width > height) {
    ratio = ((float) width) / ((float) height) ;
    nb_min_vert = 2 ;
    nb_max_vert = height / MIN_PIECE_SIZE ;
    nb_min_horiz = (int) (((float) nb_min_vert) * ratio) ;
    nb_max_horiz = (int) (((float) nb_max_vert) * ratio) ;

    *nb_resolutions = nb_max_vert - nb_min_vert ;
    resolutions = malloc ((*nb_resolutions) * sizeof (struct resolution)) ;
    if (resolutions == NULL) {
      nb_resolutions = 0 ;
      return (NULL) ;
    }
    
    j = 0 ;
    for (i = nb_min_vert; i < nb_max_vert; i++) {
      int h = (int) (((float) i) * ratio) ;
      resolutions[j].nb_horizontally = h ;
      resolutions[j].nb_vertically = i ;
      resolutions[j].average_piece_size = height / i ;
      j++ ;
    }      /* End of for (i = nb_min_vert; i < nb_max_vert; i++). */
  }        /* End of if (width > height). */
  else {
    ratio = ((float) height) / ((float) width) ;
    nb_min_horiz = 2 ;
    nb_max_horiz = width / MIN_PIECE_SIZE ;
    nb_min_vert = (int) (((float) nb_min_horiz) * ratio) ;
    nb_max_vert = (int) (((float) nb_max_horiz) * ratio) ;
    
    *nb_resolutions = nb_max_horiz - nb_min_horiz ;
    resolutions = malloc ((*nb_resolutions) * sizeof (struct resolution)) ;
    if (resolutions == NULL) {
      nb_resolutions = 0 ;
      return (NULL) ;
    }
    
    j = 0 ;
    for (i = nb_min_horiz; i < nb_max_horiz; i++) {
      int v = (int) (((float) i) * ratio) ;
      resolutions[j].nb_horizontally = i ;
      resolutions[j].nb_vertically = v ;
      resolutions[j].average_piece_size = width / i ;
      j++ ;
    }      /* End of for (i = nb_min_horiz; i < nb_max_horiz; i++). */
  }        /* End of else. */

  return (resolutions) ;
}
