/*
  JigPlayViewController.m

  Created by François Pessaux on 09/10/10.
  Copyright 2010. All rights reserved.
*/

#import "JigPlayViewController.h"

#import "IntroView.h"
#import "SetupView.h"
#import "FramePlayAreaView.h"
#import "PlayAreaView.h"
#import "HelpView.h"
#import "ColorPickerView.h"


@implementation JigPlayViewController

@synthesize imgPickerController ;
@synthesize framePlayAreaView ;

#pragma mark -
#pragma mark Initialization

- (void) awakeFromNib
{
  /* Initially, set the contained view to be the intro one. Note that since we
     don't call loadView, no method like viewWillAppear, viewDidLoad, and so on
     are called. Instead, we directly set the view instance variable of the
     controler. Note that in our NOB, we didn't set this instance variable: we
     do this here manually instead. */
  self.view = introView ;
  self.imgPickerController = [[UIImagePickerController alloc] init] ;
  self.imgPickerController.allowsEditing = NO ;
  self.imgPickerController.delegate = self ;
  self.imgPickerController.sourceType = UIImagePickerControllerSourceTypePhotoLibrary ;
}



#pragma mark -
#pragma mark View Showing and Hidding

- (void) showIntroView
{
	self.view = introView ;
}



- (void) showSetupView
{
	self.view = setupView ;
}



- (void) showFramePlayAreaView
{
  self.view = framePlayAreaView ;
}



- (void) startNewGameAndShowPlayAreaView: (UIImage*) image
                         withHorizPieces: (int) nb_horiz_pieces
                          withVertPieces: (int) nb_vert_pieces
                         withScaleFactor: (float) scale_factor
                         withWindowWidth: (int) width
                        withWindowHeight: (int) height
{
	[self showFramePlayAreaView] ;
  PlayAreaView *plView = [framePlayAreaView getPlayAreaView] ;
  plView.delaysContentTouches = NO ;
  [plView startNewGame: image
     withHorizPieces: nb_horiz_pieces
      withVertPieces: nb_vert_pieces
     withScaleFactor: scale_factor
     withWindowWidth: width
    withWindowHeight: height] ;
}



- (void) showHelpView
{
	self.view = helpView ;
  [helpView loadHelpHTML] ;
}



- (void) showColorPickerView
{
  self.view = colorPickerView ;
}



#pragma mark -
#pragma mark Autorotate Handling

/** Overidden to allows change between portrait and landscape when the device
    it rotated. Currently, imperfect since this should only be enabled when in
    the play view. To do better later.
    This works fine because under IB we tagged our PlayAreaView to resize in
    both x and y and to keep its margins ratios ("Autosizing" section) . */
- (BOOL) shouldAutorotateToInterfaceOrientation:
    (UIInterfaceOrientation) interfaceOrientation
{
  /* Allow changing the orientation only is the PlayAreaView is the visible
     one. */
  if (self.view != framePlayAreaView) return (FALSE) ;
  
  /* Return YES for supported orientations. */
  return
    ((interfaceOrientation == UIInterfaceOrientationPortrait) ||
     (interfaceOrientation == UIInterfaceOrientationLandscapeRight) ||
     (interfaceOrientation == UIInterfaceOrientationLandscapeLeft) ||
     (interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown)) ;
}



#pragma mark -
#pragma mark Callback for image selection (better if in SettingsView) 

- (IBAction) selectImagePressed: (id) sender
{
  [self presentModalViewController: self.imgPickerController animated: YES] ;
}



#pragma mark -
#pragma mark UIImagePickerControllerDelegate Protocol Handling

- (void) imagePickerController: (UIImagePickerController*) picker
         didFinishPickingImage: (UIImage*) image
                   editingInfo: (NSDictionary*) editInfo
{
  [[picker parentViewController] dismissModalViewControllerAnimated: YES] ;
  /* Notify the SetupView that the image changed, hence the GUI must be
     updated. */
  [setupView imageSelectedCallback: image] ;
}



#pragma mark -
#pragma mark Memory Management

- (void) didReceiveMemoryWarning
{
  [super didReceiveMemoryWarning] ; // Releases the view if it doesn't have a superview
  // Release anything that's not essential, such as cached data
}


- (void) dealloc
{
  [super dealloc] ;
}

@end
